%% Simulation 1: bootstrap
%% note that my_true_model.m generate the signal
%my_true_model.m contains the data generating process, which was used in my_conf_int_test_fft
%check my_true_model.m for which model is the simulation using.
%%
clear all; clc; close all;
%% Length of Time Series
N=400; %num of data (e.g. 400 or 800)
%% Setting n and BnFFT
n=54; % n and BnFFT have been chosen already 
BnFFT=32; %BnFFT<n
%%

Bn=floor(BnFFT/2); % Bn: the effective number of frequencies is half of BnFFT+1 (discarded the last one)
if BnFFT>n
    disp('error: BnFFT>n');
    return;
end
%% Generating u
du=n/N*(1-1/(log(Bn))^2); % du: fixed interval size between time
u=[n/2/N:du:1-n/2/N]'; % u: the vector of times 
Cn=size(u,1); % Cn: number of time points

%% Monte Carlo Setting
myplots=0; % don't plot figures when myplot=0;


per1=0.95; % per: 95% or 90%
per2=0.9;
MonteCarlo1=10000; % MonteCarlo1: number of Monte Carlo for generating bootstrapped interval
MonteCarlo2=1000; % MonteCarlo2: number of Monte Carlo for testing



%% Monte Carlo 1: generate bootstrapped interval

[int1_l,int1_r,int2_l,int2_r,boot_maxdev,boot_avg]=my_bootstrap_fft(u,N,n,BnFFT,Bn,MonteCarlo1,per1,per2);

disp('generated interval by bootstrapping: ');
[int1_l,int1_r]
[int2_l,int2_r]
%% Monte Carlo 2: test interval (Generating data from Model: X[i]=0.3*cos(2*pi*(i/N))X[i-1]+e[i])

[my_per1,my_per2,test_maxdev,test_avg]=my_conf_int_test_fft(u,N,n,BnFFT,Bn,MonteCarlo2,int1_l,int1_r,int2_l,int2_r);

disp(' confidence by testing: ');
[1-my_per1,1-my_per2]
%per-my_per

%% avg size(width) of confidence regions
disp(' avg width of SCR: ');
% sqrt(intl_r) is gamma_{1-alpha} and mean(test_avg) is avg f(u,theta)
% avg width is about gamma*E(f(u,theta)) if the left region is zero, or
% 2gamma*E(f(u,theta)) if the left region is not zero
[(1+(int1_r<1))*sqrt(int1_r)*mean(test_avg),(1+(int2_r<1))*sqrt(int2_r)*mean(test_avg)] % if int1_r<1 then 2*gamma*E(f); otherwise 1*gamma*E(f)

%% check if figures are needed
if myplots==0
    break
end
%% Some plots: hist of bootstraps and testing
figure(1);hist(boot_maxdev);title('Empirical Distribution of Bootstrapped Data'); axis([0 10 0 inf]);
figure(2);hist(test_maxdev);title('Monte Carlo Experiments of Non-stationary Data');axis([0 10 0 inf]);

%% More plots: empirical density estimate for both bootstrap and testing (good performance if they almost overlap)
[f1,x1]=ksdensity(boot_maxdev);
[f2,x2]=ksdensity(test_maxdev);
%% gumbel, but not used (since needs rescale and shift)
%x3=[-5:0.1:20];
%f3=exp(-x3);
%f3=exp(-x3-f3);
%%
figure(3);plot(x1,f1,'--',x2,f2,'+');legend('Bootstrap','Test');%Gumbel need to rescale... so didn't include it..


%% Additional plots: True Densities and Estimated Densities
%% NOTE: this shoud correspond to the true model (check my_true_model.m)
phi2=0.3*cos(2*pi*u); %true model is X[i]=0.3*cos(2*pi*(i/N))X[i-1]+e[i]
%%
phi1=zeros(length(phi2),1); %reference
 theta=2*pi*[0:Bn-1]/(BnFFT);
 f1=zeros(1,length(boot_avg));
 f2=zeros(1,length(test_avg));
 for uu=1:length(phi2)
 f1((uu-1)*length(theta)+1:uu*length(theta))=1/2/pi./(1-2*phi1(uu)*cos(theta)+phi1(uu)^2); %reference
 f2((uu-1)*length(theta)+1:uu*length(theta))=1/2/pi./(1-2*phi2(uu)*cos(theta)+phi2(uu)^2); %true density
 end
 rescale1=mean(f1)/mean(boot_avg); % adjust the constant
 rescale2=mean(f2)/mean(test_avg); %rescale2 should equal rescale1
 rescale3=1/2; % fixed 0.5 for rescale 
 figure(4);  %estimated densities
 for uuu=1:min(length(u),9)
 subplot(3,3,uuu);
 plot(1:length(theta),boot_avg((uuu-1)*length(theta)+1:uuu*length(theta))*rescale3,'-.',1:length(theta),test_avg((uuu-1)*length(theta)+1:uuu*length(theta))*rescale3,'*-');axis([1,length(theta),-Inf,Inf]);
 stitle=sprintf('Est f(u=%0.2f,.)',u(uuu));
 title(stitle);
 end
 figure(5); %true densities
for uuu=1:min(length(u),9)
    subplot(3,3,uuu);
 plot(1:length(theta),f1((uuu-1)*length(theta)+1:uuu*length(theta)),'-.',1:length(theta),f2((uuu-1)*length(theta)+1:uuu*length(theta)),'*-');axis([1,length(theta),-Inf,Inf]);
  stitle=sprintf('True f(u=%0.2f,.)',u(uuu));
 title(stitle);
end
