%% Simulation 3: select n and Bn for the two examples in Simulation 1

%%
clear all; clc; close all;
%% Length Time Series (Generated from my_true_model. This part can be replaced by real time series)
N=400; %num of data (e.g. 400 or 800)
eta=0.48;
%% Setting up Monte Carlo 
MonteCarlo=100;
my_select_n=zeros(MonteCarlo,1);
my_select_b=zeros(MonteCarlo,1);


%% Setting for searching n and BnFFT
if N<1000
    n_min=round(2.5*(N^eta)); % min 1/2*sqrt(N)
    n_max=round(3.5*(N^eta)); % max 4*sqrt(N)
else 
    n_min=round(2.5*(N^eta)); %2.5
    n_max=round(4*(N^eta));    % 4
end
BnFFT_base=2^(floor(log2(n_min))); % min BnFFT is the largest 2^k that is smaller than n_min
BnFFT_min=min(round(1.5*sqrt(N)),28);
du_base=n_max/N*(1-1/(log(BnFFT_base))^2); % largest time interval
u_base=[n_max/2/N:du_base:1-n_max/2/N]'; % u: the vector of times 
f_base=2*pi*[0:floor(BnFFT_base/2)-1]'/(BnFFT_base);
Cn_base=size(u_base,1); % Cn_base: number of base time points
Bn_base=size(f_base,1); % Bn_base: number of base freq points


%% Choose n in [n_min, n_max] and BnFFT in [BnFFT, n)
%% generating n_list and BnFFT_list
n_tmp=[n_min:n_max];
b_tmp=[BnFFT_min: n_max-1];
tmp_size_n=length(n_tmp);
tmp_size_b=length(b_tmp);
tmp_list=[repmat(n_tmp,1,tmp_size_b)',vec(repmat(b_tmp,tmp_size_n,1))];
tmp_ind=tmp_list(:,1)>tmp_list(:,2); %choose BnFFT<n
my_list=tmp_list(tmp_ind,:); % generated all possible pairs
n_list=my_list(:,1); % the list of n
b_list=my_list(:,2); % the list of BnFFT
tot_pair=length(n_list); % total num of pairs


%% computing density estimates for all pairs
for mmm=1:MonteCarlo
    my_estimates=zeros(Bn_base*Cn_base,tot_pair); % final results 
    %% generated time series
    X=my_true_model(N); % generated X
    %% start computing...
    for iii=1:tot_pair
      %  iii/tot_pair %show progress

        n=n_list(iii); % n and BnFFT have been chosen already 
        BnFFT=b_list(iii); %BnFFT<n

        %if BnFFT>n %tmp debug, remove later
        %    disp('error: BnFFT>n');
        %    return;
        %end

        %% Generating u_vec and f_vec
        du=n/N*(1-1/(log(BnFFT))^2); % du: fixed interval size between time
        u_vec=[n/2/N:du:1-n/2/N]'; % u_vec: the vector of times 
        Cn=size(u_vec,1); % Cn: number of time points
        Bn=floor(BnFFT/2); % Bn: the effective number of frequencies is half of BnFFT+1 (discarded the last one)
        f_vec=2*pi*[0:Bn-1]'/(BnFFT); %f_vec: the vector of freq

        %% For selected time points, compute the FFT then choose the subset of selected frequencies
        temp_res=zeros(Cn_base,Bn_base);
        %Note: later save as use vec(temp_res);
        %%
        u_ind=nearestpoint(u_base,u_vec); % match u_base by nearest u in u_vec
        my_u=u_vec(u_ind); % vector of adjusted time points (from u_vec, same length as u_base)
        f_ind=nearestpoint(f_base,f_vec); % match f_base by nearest freq in f_vec, indices are enough
        for jj=1:Cn_base
            uu=my_u(jj); 
            f_temp=my_spectral_estimate_fft(X,n,BnFFT,Bn,uu)';
            temp_res(jj,:)=f_temp(f_ind);
        end

        %% save to my_estimates
        my_estimates(:,iii)=vec(temp_res); %save as a vector, can be reshaped back by reshape(my_estimates(:,iii), Cn_base, Bn_base)
    end

    %% computing variance
    temp_var=(my_estimates-repmat(mean(my_estimates,2),1,tot_pair)).^2;
    my_var=sum(temp_var,1);
    pair_ind=find(my_var==min(my_var));
    final_ind=pair_ind(1); %in case more than one
    %n_selected=n_list(final_ind);
    %b_selected=b_list(final_ind);
    mmm/MonteCarlo
    n_list(final_ind)
    b_list(final_ind)
    my_select_n(mmm)=n_list(final_ind);
    my_select_b(mmm)=b_list(final_ind);
    
    % disp('selected n and BnFFT:');
   % my_list(final_ind,:)

%     du_final=n_selected/N*(1-1/(log(b_selected))^2); % du: fixed interval size between time
%     u_vec_final=[n/2/N:du_final:1-n/2/N]'; % u_vec: the vector of times 
%     c_selected=size(u_vec_final,1); % Cn: number of time points
%     disp('corresponding Bn and Cn:');
%     [floor(b_selected/2),c_selected]
%     %% other choices
%     more_pair_ind=find(my_var<=1.2*min(my_var));
%     disp('more choices of (n,BnFFT) and their total vars (<=1.2*min):');
%     [my_list(more_pair_ind,:),my_var(more_pair_ind)']
end    
avg_n=mean(my_select_n);
avg_b=mean(my_select_b);
disp('Avg selection of (n and Bn):')
avg_n
avg_b