clear all; close all; clc;
%% setting for time-varying AR, or use SP500 abs (removed mean)
    %N=6779;
    %n=314;
    %% adjust the parameters of the model here
    c0=0.3;
    c1=0.2;
    c2=0.3;
    c3=0.2;
    MonteCarlo2=1000; % small to speed up
    %%
    %BnFFT=32;
    N=800;
    n=72;
    BnFFT=36;
    Bn=floor(BnFFT/2);
    du=n/N*(1-1/(log(Bn))^2); % du: fixed interval size between time
    u=[n/2/N:du:1-n/2/N]'; % u: the vector of times 
    Cn=size(u,1); % Cn: number of time points
    
    %% bootstrap
    MonteCarlo=10000;
    [int1_l,crit1,int2_l,crit2,mydata_maxdev,mydata_avg]=my_bootstrap_fft(u,N,n,BnFFT,Bn,MonteCarlo,0.95,0.9);
    %load('maxdev_sp500.mat'); %mydata_maxdev for SP500 size
    %
    %% start monte carlo
    
    counter1=0;
    counter2=0;
    counter3=0;
    counter4=0;
    for kkk=1:MonteCarlo2
        kkk/MonteCarlo2 %show progress
    %% generate tv-AR
    epsilon=normrnd(0,1,N,1); %epsilon
    %X_temp=normrnd(0,1,N,1);
    X=epsilon;
    %%
    for iii=2:N 
        %% the time-varying AR(1) model a1=0.3cos(2pi*u)  sigma=(1+u+u^2)
        %% if generation part changed then note that theoretical part also need to be changed 
        %X(iii)=(0.4*cos(2*pi*iii/N))*X(iii-1)+(1+0.3*iii/N+0.2*(iii/N)^2)*epsilon(iii);    
        X(iii)=(c0+c1*iii/N)*X(iii-1)+(1+c2*iii/N+c3*(iii/N)^2)*epsilon(iii);    
    end
    X=X-mean(mean(X)); 
    %%
    f_nonpara=zeros(Cn,Bn);
    f_local_period=zeros(Cn,Bn);
    f_local_para=zeros(Cn,Bn);
    f_para=zeros(Cn,Bn);
    f_theory=zeros(Cn,Bn);
    p_for_AR=1; %AR(p)
    local_fit=zeros(Cn,p_for_AR+1); %[sigma, AR(p)] = size p+1
    %% parametric and nonparametric grid
    for ii=1:Cn
        my_u=u(ii);
        %nonparametric grid
        f_nonpara(ii,:)=my_spectral_estimate_fft(X,n,BnFFT,Bn,my_u)';
        f_local_period(ii,:)=my_local_periodogram_fft(X,n,BnFFT,Bn,my_u)';
        %parametric grid
        %% a1 and sigma is using 1+0.2u and 1+u+u^2
        %% theoretical part may be changed if the generation part changed
        a1=-(c0+c1*my_u); sigma=1+c2*my_u+c3*my_u^2;%(0.7-0.3*my_u);
        %a1=-0.3; sigma=0.3;
        %f_theory(ii,:)=sigma^2/2/pi./(abs(1+a1*exp(sqrt(-1)*2*pi.*[0:Bn-1]./BnFFT)).^2);
        f_theory(ii,:)=my_AR_density(sigma,a1,BnFFT,Bn);
        %% fit AR using local whittle
        % define local optimization objective
        %f_local = @(my_para)local_whittle_AR_p(my_para,p_for_AR,BnFFT,Bn,f_local_period(ii,:));
        %% log local whittle (plus some constant to make sure it's positive before logarithm
        f_local = @(my_para)log_local_whittle_AR_p(my_para,p_for_AR,BnFFT,Bn,f_local_period(ii,:));
        %f_local = @(my_para)log_local_whittle_AR_p(my_para,p_for_AR,BnFFT,Bn,f_nonpara(ii,:));
        % optimization
        [my_opt_para,fval]=fminsearch(f_local,[sigma,a1]);
        % compute local parametric density
        f_local_para(ii,:)=my_AR_density(my_opt_para(1),my_opt_para(2),BnFFT,Bn);
        % save local parameters
        local_fit(ii,:)=my_opt_para; %store 
    end
    %% fit globally by poly
    my_sigma=local_fit(:,1);%note that poly order 2 for sigma
    my_a1=local_fit(:,2);%note that poly order 1 for a1
    poly_para_sigma=polyfit(u,my_sigma,2); % poly fit sigma using 1+u+u^2
    poly_para_a1=polyfit(u,my_a1,1); % poly fit a1 using 1+u
    my_fitted_sigma=polyval(poly_para_sigma,u);
    my_fitted_a1=polyval(poly_para_a1,u);
    
    for iii=1:Cn
        f_para(iii,:)=my_AR_density(my_fitted_sigma(iii),my_fitted_a1(iii),BnFFT,Bn);
    end
     res1=(f_para./f_nonpara-1).^2; % this is confidence region
     %res2=(f_nonpara./f_para-1).^2; % if f_para is theoretical then this is our theorem
     res0=(f_local_para./f_nonpara-1).^2;% for semi-para
     res=res1;
    my_largest=max(max(res));
    my_largest0=max(max(res0));
    %find(res==my_largest);
    
    
    %% 
    my_pval=sum(mydata_maxdev>my_largest)/length(mydata_maxdev);
    counter1=counter1+(my_pval<0.05);
    counter2=counter2+(my_pval<0.1);
    
    my_pval0=sum(mydata_maxdev>my_largest0)/length(mydata_maxdev);
    counter3=counter3+(my_pval0<0.05);
    counter4=counter4+(my_pval0<0.1);
    end
    % for fully para
    my_accurate1=counter1/MonteCarlo2
    my_accurate2=counter2/MonteCarlo2
    % for semi-para
    my_accurate3=counter3/MonteCarlo2
    my_accurate4=counter4/MonteCarlo2
    
    figure(1); mesh(f_theory'); title('theoretical');
    figure(2); mesh(f_nonpara'); title('nonparametric');
    figure(3); mesh(f_local_para'); title('local parametric');
    figure(4); mesh(f_para'); title('parametric');
    figure(5); mesh(f_local_period'); title('local periodogram');
    %figure(3); mesh(res1); title('normalized max dev, f/fhat');
    %figure(4); mesh(res2); title('normalized max dev, fhat/f');