function [n_selected,b_selected]=my_select_n_BnFFT(X)
%% need to carefully set n_min and n_max as well as the search region of (BnFFT,n)
%right now I choose BnFFT<n/log(n) when N>5000
%otherwise (N<5000) only choose BnFFT<n
%%
N=length(X);
%% Setting for searching n and BnFFT
%n_min=round(sqrt(N)); % min 1/2*sqrt(N)?
%n_max=round(4*sqrt(N)); % max 4*sqrt(N)?
n_min=round(2*N^(0.47)); % min 1/2*sqrt(N)?
n_max=round(3*N^(0.47));
if N>2000
    n_min=round(3*N^(0.47));
    n_max=round(5*N^(0.47));
end
%
BnFFT_base=2^(floor(log2(n_min))); % min BnFFT is the largest 2^k that is smaller than n_min
if BnFFT_base>32
    BnFFT_base=32; %min 16
end
BnFFT_min=BnFFT_base;
du_base=n_max/N*(1-1/(log(BnFFT_base))^2); % largest time interval
u_base=[n_max/2/N:du_base:1-n_max/2/N]'; % u: the vector of times 
f_base=2*pi*[0:floor(BnFFT_base/2)-1]'/(BnFFT_base);
Cn_base=size(u_base,1); % Cn_base: number of base time points
Bn_base=size(f_base,1); % Bn_base: number of base freq points


%% Choose n in [n_min, n_max] and BnFFT in [BnFFT_min, n) but 2*log(BnFFT)*BnFFT/n<1 roughtly BnFFT<n/log(n)
%% generating n_list and BnFFT_list
n_tmp=[n_min:n_max];
b_tmp=[BnFFT_min: n_max-1];
tmp_size_n=length(n_tmp);
tmp_size_b=length(b_tmp);
tmp_list=[repmat(n_tmp,1,tmp_size_b)',vec(repmat(b_tmp,tmp_size_n,1))];
%% search region for (BnFFT,n)
%tmp_ind=tmp_list(:,1)>tmp_list(:,2); %choose BnFFT<n 
%tmp_ind=tmp_list(:,1)>(2*log(tmp_list(:,2)).*tmp_list(:,2)); %choose 2log(BnFFT)BnFFT<n
if N>2000
    tmp_ind=tmp_list(:,1)./log(tmp_list(:,1))>tmp_list(:,2); %choose BnFFT<n/log(n)
else
    tmp_ind=tmp_list(:,1)>tmp_list(:,2); %choose BnFFT<n 
end
%%
my_list=tmp_list(tmp_ind,:); % generated all possible pairs
n_list=my_list(:,1); % the list of n
b_list=my_list(:,2); % the list of BnFFT
tot_pair=length(n_list); % total num of pairs
%% computing density estimates for all pairs
my_estimates=zeros(Bn_base*Cn_base,tot_pair); % final results 

%start computing...
for iii=1:tot_pair
    iii/tot_pair %show progress
    
    n=n_list(iii); % n and BnFFT have been chosen already 
    BnFFT=b_list(iii); %BnFFT<n

    %if BnFFT>n %tmp debug, remove later
    %    disp('error: BnFFT>n');
    %    return;
    %end

    %% Generating u_vec and f_vec
    du=n/N*(1-1/(log(BnFFT))^2); % du: fixed interval size between time
    u_vec=[n/2/N:du:1-n/2/N]'; % u_vec: the vector of times 
    Cn=size(u_vec,1); % Cn: number of time points
    Bn=floor(BnFFT/2); % Bn: the effective number of frequencies is half of BnFFT+1 (discarded the last one)
    f_vec=2*pi*[0:Bn-1]'/(BnFFT); %f_vec: the vector of freq

    %% For selected time points, compute the FFT then choose the subset of selected frequencies
    temp_res=zeros(Cn_base,Bn_base);
    %Note: later save as use vec(temp_res);
    %%
    u_ind=nearestpoint(u_base,u_vec); % match u_base by nearest u in u_vec
    my_u=u_vec(u_ind); % vector of adjusted time points (from u_vec, same length as u_base)
    f_ind=nearestpoint(f_base,f_vec); % match f_base by nearest freq in f_vec, indices are enough
    for jj=1:Cn_base
        uu=my_u(jj); 
        f_temp=my_spectral_estimate_fft(X,n,BnFFT,Bn,uu)';
        temp_res(jj,:)=f_temp(f_ind);
    end

    %% save to my_estimates
    my_estimates(:,iii)=vec(temp_res); %save as a vector, can be reshaped back by reshape(my_estimates(:,iii), Cn_base, Bn_base)
end

%% computing variance
temp_var=(my_estimates-repmat(mean(my_estimates,2),1,tot_pair)).^2;
my_var=sum(temp_var,1);
pair_ind=find(my_var==min(my_var));
final_ind=pair_ind(1); %in case more than one
n_selected=n_list(final_ind);
b_selected=b_list(final_ind);
disp('selected n and BnFFT:');
my_list(final_ind,:)

du_final=n_selected/N*(1-1/(log(b_selected))^2); % du: fixed interval size between time
u_vec_final=[n/2/N:du_final:1-n/2/N]'; % u_vec: the vector of times 
c_selected=size(u_vec_final,1); % Cn: number of time points

%% other choices
more_pair_ind=find(my_var<=1.2*min(my_var));
disp('more choices of (n,BnFFT) and their total vars (<=1.2*min):');
[my_list(more_pair_ind,:),my_var(more_pair_ind)']

%%
disp('corresponding Bn and Cn:');
[floor(b_selected/2),c_selected]